#
# spec file for package docker
#
# Copyright (c) 2019 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#
# nodebuginfo


# Handle _multibuild magic.
%define flavour %{nil}

# We split the Name: into "realname" and "name_suffix".
%define realname docker-runc
%if "%flavour" == ""
%define name_suffix %{nil}
%else
%define name_suffix -%{flavour}
%endif

# MANUAL: Update the git_version, git_short, and git_revision
%define git_version 2b18fe1d885ee5083ef9f0838fee39b62d653e30
%define git_short   2b18fe1d885e
# How to get the git_revision
# git clone ${url}.git runc-upstream
# cd runc-upstream
# git checkout $git_version
# git_revision=r$(git rev-list HEAD | wc -l)
%define git_revision r3804

%define go_tool go
%define _name runc
%define project github.com/opencontainers/%{_name}

Name:           %{realname}%{name_suffix}
Version:        1.0.0rc6+git%{git_revision}_%{git_short}
Release:        lp151.3.3.1
Summary:        Tool for spawning and running OCI containers
License:        Apache-2.0
Group:          System/Management
Url:            https://github.com/opencontainers/runc
Source:         %{realname}-git.%{git_version}.tar.xz
Source1:        %{realname}-rpmlintrc
BuildRequires:  fdupes
BuildRequires:  go-go-md2man
BuildRequires:  libapparmor-devel
BuildRequires:  libseccomp-devel >= 2.2
BuildRequires:  libselinux-devel
BuildRequires:  golang(API) >= 1.10
Recommends:     criu
Obsoletes:      runc <= 1.0
# We provide a git revision so that Docker can require it properly.
Provides:       %{name}-git = %{git_version}
ExcludeArch:    s390
# KUBIC-SPECIFIC: This was required when upgrading from the original kubic
#                 packaging, when everything was renamed to -kubic. It also is
#                 used to ensure that nothing complains too much when using
#                 -kubic packages. Hopfully it can be removed one day.
%if "%flavour" == "kubic"
# Obsolete older package without -kubic suffix: v2 -> v3
Obsoletes:      %{_name}_50a19c6
Obsoletes:      %{realname} = 0.1.1+gitr2819_50a19c6
# Conflict with non-kubic package, and provide equivalent
Conflicts:      %{realname}
Provides:       %{realname} = %{version}
# Disable leap based builds for kubic flavor (bsc#1121412)
%if 0%{?suse_version} == 1500 && 0%{?is_opensuse}
ExclusiveArch:  do_not_build
%endif
%endif

%description
runc is a CLI tool for spawning and running containers according to the OCI
specification. It is designed to be as minimal as possible, and is the workhorse
of Docker. It was originally designed to be a replacement for LXC within Docker,
and has grown to become a separate project entirely.

%package test
Summary:        Test package for runc
Group:          System/Management
BuildRequires:  golang(API) = 1.10
Requires:       go-go-md2man
Requires:       libapparmor-devel
BuildRequires:  libseccomp-devel >= 2.3
Requires:       libselinux-devel
Recommends:     criu
BuildArch:      noarch
Obsoletes:      runc-test <= 1.0
# KUBIC-SPECIFIC: This was required when upgrading from the original kubic
#                 packaging, when everything was renamed to -kubic. It also is
#                 used to ensure that nothing complains too much when using
#                 -kubic packages. Hopfully it can be removed one day.
%if "%flavour" == "kubic"
# Obsolete older package without -kubic suffix: v2 -> v3
Obsoletes:      %{realname}-test = 0.1.1+gitr2819_50a19c6
# Conflict with non-kubic package, and provide equivalent
Conflicts:      %{realname}-test
Provides:       %{realname}-test = %{version}
%endif

%description test
Test package for runc. It contains the source code and the tests.

%prep
%setup -q -n %{realname}-git.%{git_version}

%build
# Do not use symlinks. If you want to run the unit tests for this package at
# some point during the build and you need to directly use go list directly it
# will get confused by symlinks.
export GOPATH=${HOME}/go
export PROJECT=${HOME}/go/src/%project
mkdir -pv $PROJECT
rm -rf $PROJECT/*
cp -av * $PROJECT

# Build all features.
export BUILDTAGS="apparmor selinux seccomp"
export BUILDFLAGS="-buildmode=pie -gccgoflags='-Wl,--add-needed -Wl,--no-as-needed -static-libgo -ldl -lselinux -lapparmor -lseccomp'"

(cat <<EOF
export GOPATH="$GOPATH"
export BUILDTAGS="$BUILDTAGS"
export BUILDFLAGS="$BUILDFLAGS"
EOF
) >./.runc_build_env
source ./.runc_build_env

# Build runc.
make -C $PROJECT EXTRA_FLAGS="-x $BUILDFLAGS" BUILDTAGS="$BUILDTAGS" COMMIT_NO=%{git_version} runc
cp $PROJECT/runc %{realname}-%{version}

# Build man pages, this can only be done on arches where we can build go-md2man.
man/md2man-all.sh

%check
# We used to run 'go test' here, however we found that this actually didn't
# catch any issues that were caught by smoke testing, and %check would
# continually cause package builds to fail due to flaky tests. If you ever need
# to know how the testing was done, you can always look in the package history.
# boo#1095817

%install
source ./.runc_build_env

# Make sure we install in /usr/sbin/docker-runc
install -D -m755 %{realname}-%{version} %{buildroot}%{_sbindir}/%{realname}
install -d -m755 %{buildroot}/usr/src/%{realname}/
cp -av $HOME/go/src/%{project}/* %{buildroot}/usr/src/%{realname}/
rm -rf %{buildroot}/usr/src/docker-runc/runc

# We have to rename the man pages to docker-runc.
install -d -m755 %{buildroot}%{_mandir}/man8
cd man/man8
for mp in $(ls runc*.8); do
	install -m644 ${mp} %{buildroot}%{_mandir}/man8/docker-${mp}
done

%fdupes %{buildroot}

%files
%defattr(-,root,root)
%doc README.md
%license LICENSE
%{_sbindir}/docker-runc
%{_mandir}/man8/docker-runc*.8.gz

%files test
%defattr(-,root,root)
/usr/src/docker-runc/

%changelog
* Fri May  3 2019 Aleksa Sarai <asarai@suse.com>
- Update to runc 2b18fe1d885e, which is required for Docker v18.09.5-ce.
  bsc#1128376 boo#1134068
- Remove patches which were merged upstream:
  - bsc1131553-0001-nsenter-cloned_binary-various-cleanups.patch
* Wed Apr 24 2019 Aleksa Sarai <asarai@suse.com>
- Backport various upstream patches to fix some kernel regression related to
  O_TMPFILE. bsc#1131314 bsc#1131553
  * bsc1131553-0001-nsenter-cloned_binary-various-cleanups.patch
* Fri Mar 22 2019 Sascha Grunert <sgrunert@suse.com>
- Update to runc 6635b4f0c6af, which is required for Docker v18.09.3-ce.
- Remove patches that were merged upstream:
  * CVE-2019-5736.patch
* Wed Feb  6 2019 Aleksa Sarai <asarai@suse.com>
- Add fix for CVE-2019-5736 (effectively copying /proc/self/exe during re-exec
  to avoid write attacks to the host runc binary). bsc#1121967
  + CVE-2019-5736.patch
- Add docker-runc-rpmlintrc for docker-runc-test.
* Tue Feb  5 2019 Aleksa Sarai <asarai@suse.com>
- Update to runc 96ec2177ae84, which is required for Docker v18.09.1-ce.
  bsc#1124308
  This contains changes for rc6:
  * https://github.com/opencontainers/runc/releases
  Plus additional changes:
  * may kill other process when container has been stopped
  (https://github.com/opencontainers/runc/commit/87a188996e229bf382c27865584765d1a50c021)
  * kill: allow to signal paused containers:
  (https://github.com/opencontainers/runc/commit/07d1ad44c83c4274f01e2db18776f31b4dd8e13c)
  * Modify check-config.sh in accordance with Moby Project updates
  (https://github.com/opencontainers/runc/commit/30817421efbc761c0adcb1d67b8ef84ae32171be)
  * cr: get pid from criu notify when restore
  (https://github.com/opencontainers/runc/commit/dce70cdff53f576e1334de2a545326599ec824a6)
* Fri Jan 11 2019 Sascha Grunert <sgrunert@suse.com>
- Disable leap based builds for kubic flavor. bsc#1121412
* Wed Dec 19 2018 clee@suse.com
- Update go requirements to >= go1.10 to fix
  * bsc#1118897 CVE-2018-16873
    go#29230 cmd/go: remote command execution during "go get -u"
  * bsc#1118898 CVE-2018-16874
    go#29231 cmd/go: directory traversal in "go get" via curly braces in import paths
  * bsc#1118899 CVE-2018-16875
    go#29233 crypto/x509: CPU denial of service
* Thu Aug 16 2018 asarai@suse.com
- Merge -kubic packages back into the main Virtualization:containers packages.
  This is done using _multibuild to add a "kubic" flavour, which is then used
  to conditionally compile patches and other kubic-specific features.
  bsc#1105000
* Wed Aug  1 2018 asarai@suse.com
- Enable seccomp support on SLE12, since libseccomp is now a new enough vintage
  to work with Docker and containerd. fate#325877
* Wed Jul 25 2018 asarai@suse.com
- Upgrade to docker-runc 69663f0bd4b60df09991c08812a60108003fa340 which is
  required for the Docker v18.06.0-ce upgrade. bsc#1102522 bsc#1113313
- Switch to Go 1.10 for building.
- Removed the following patches because they were merged upstream in the
  meantime:
  - bsc1094680-0001-Avoid-race-when-opening-exec-fifo.patch
  - bsc1053532-0001-makefile-drop-usage-of-install.patch
* Thu Jun  7 2018 asarai@suse.com
- Backport of https://github.com/opencontainers/runc/pull/1698 to help fix
  bsc#1094680, which is caused by the race described in the upstream issue.
  * bsc1094680-0001-Avoid-race-when-opening-exec-fifo.patch
* Tue Jun  5 2018 dcassany@suse.com
- Make use of %%license macro
* Tue Jun  5 2018 asarai@suse.com
- Remove 'go test' from %%check section, as it has only ever caused us problems
  and hasn't (as far as I remember) ever caught a release-blocking issue. Smoke
  testing has been far more useful. boo#1095817
* Wed May 16 2018 jmassaguerpla@suse.com
- Fix Obsoletes to fix bsc#1080978
* Mon Feb 12 2018 rbrown@suse.com
- Add ${version} to equivalent non-kubic package provides
* Thu Feb  8 2018 rbrown@suse.com
- Add Provides for equivalent non-kubic packages
* Thu Feb  1 2018 ro@suse.de
- do not build on s390, only on s390x (no go on s390)
* Wed Dec 27 2017 dimstar@opensuse.org
- Fix build with RPM 4.14: exclude is not meant for files to NOT be
  packaged, but should only be used if the files are to be excluded
  from a glob when they end up in a different package. Rather
  remove the unwanted files in the install section.
* Mon Dec 11 2017 asarai@suse.com
- Update to docker-runc 0351df1c5a66838d0c392b4ac4cf9450de844e2d requirement
  for Docker v17.09.1-ce. This also includes a switch to the upstream
  opencontainers sources.
* Mon Oct 16 2017 asarai@suse.com
- Drop backport of https://github.com/opencontainers/runc/pull/1603. bsc#1055676
  It's going to be fixed in Docker instead.
  - bsc1055676-0001-rootfs-preserve-old-mount-flags-when-remounting-bind.patch
* Mon Oct  9 2017 asarai@suse.com
- Update to docker-runc 2d41c047c83e09a6d61d464906feb2a2f3c52aa4, which is
  required for Docker v17.07-ce.
- Add backport of https://github.com/opencontainers/runc/pull/1603. bsc#1055676
  + bsc1055676-0001-rootfs-preserve-old-mount-flags-when-remounting-bind.patch
- Remove fix for CVE-2016-9962, as the patches are now included in the upstream
  source. bsc#1012568
  - CVE-2016-9962.patch
* Tue Sep  5 2017 thipp@suse.de
- Update bsc1053532-0001-makefile-drop-usage-of-install.patch
  + replace missing target "all" with "runc"
* Sun Aug 13 2017 asarai@suse.com
- Use the upstream Makefile, to ensure that we always include the version and
  commit information in runc. This was confusing users (and Docker).
  bsc#1053532
- Add a backported patch to fix a Makefile bug. This also includes some other
  changes to make the docker-runc.spec mirror the newer runc.spec (which
  required additional patching to the Makefile).
  https://github.com/opencontainers/runc/pull/1555
  + bsc1053532-0001-makefile-drop-usage-of-install.patch
* Wed Aug  2 2017 asarai@suse.com
- Use -buildmode=pie for tests and binary build. bsc#1048046 bsc#1051429
- Cleanup seccomp builds similar to bsc#1028638
- Remove the usage of 'cp -r' to reduce noise in the build logs.
* Thu Jul 13 2017 thipp@suse.de
- Remove empty %%post
- Use Obsoletes instead of Conflicts
* Thu Jul  6 2017 thipp@suse.de
- rename runc to docker-runc (bsc#1069758)
* Thu May  4 2017 jmassaguerpla@suse.com
- fix the golang requirement to 1.7 to the subpackages
* Tue May  2 2017 jmassaguerpla@suse.com
- fix golang requirement to 1.7
* Fri Apr 28 2017 jengelh@inai.de
- Substitute %%__-type macro indirections
* Thu Apr 13 2017 jmassaguerpla@suse.com
- update version to the one required by docker-17.04.0-ce (bsc#1034053)
  remove ignore_cgroup2_mountpoint.patch . This is already included in
  the upstream source code.
* Wed Apr 12 2017 jmassaguerpla@suse.com
- Make sure this is being built with go 1.7
* Tue Apr 11 2017 jmassaguerpla@suse.com
- remove the go_arches macro because we are using go1.7 which
  is available in all archs
* Wed Mar 29 2017 jmassaguerpla@suse.com
- fix bsc#1028113 - runc: make sure to ignore cgroup v2 mountpoints
  This is a backport of https://github.com/opencontainers/runc/pull/1266
  + ignore_cgroup2_mountpoint.patch
* Fri Feb 24 2017 jmassaguerpla@suse.com
- update to docker-1.13.0 requirement
* Fri Jan 13 2017 jmassaguerpla@suse.com
- fix CVE-2016-9962 bsc#1012568 and applying the patch
  CVE-2016-9962.patch, because 1.12.6 partially fixes it (it contains
  the first patch attached in bsc#1012568)
* Mon Dec 19 2016 jmassaguerpla@suse.com
- update runc to the version used in docker 1.12.5 (bsc#1016307).
  This fixes bsc#1015661
* Mon Dec 19 2016 asarai@suse.com
- For the moment, we have to switch to using Docker's fork of runC. This *will*
  be solved properly by creating a new package purely for Docker's runC fork,
  because it's quite silly to tie OCI project releases to Docker's vendoring
  scheme. Once this is fixed, this package will be switch to being purely-OCI.
* Fri Dec 16 2016 jmassaguerpla@suse.com
- add the /usr/bin/docker-run symlink to partially fix bsc#1015661
* Thu Nov 24 2016 jmassaguerpla@suse.com
- fix version by adding a revision "counter" so that it will always
  increase
  fix bsc#1009961
* Thu Oct 13 2016 jmassaguerpla@suse.com
- update to 02f8fa7 because that is the needed version for docker 1.12.1 (bsc#1004490)
* Wed Sep 21 2016 jengelh@inai.de
- Run fdupes.
* Mon Sep 19 2016 jmassaguerpla@suse.com
- fix go_arches definition: use global instead of define, otherwise
  it fails to build
* Fri Aug 26 2016 asarai@suse.com
- Remove docker-runc symlink because it's been fixed within the Docker
  package. bsc#978260
* Thu Aug 25 2016 jmassaguerpla@suse.com
- Create a symlink /usr/sbin/docker-runc -> /usr/sbin/docker
  Docker expects this symlink to exist bsc#978260
* Thu Aug 25 2016 jmassaguerpla@suse.com
- Remove GOPATH at the end of the GOPATH assignment
  cause GOPATH is empty and if we do that, we get the path ""
  appended, which causes gcc6-go to complain
* Wed Aug 24 2016 jmassaguerpla@suse.com
- add go_arches in project configuration: this way, we can use the
  same spec file but decide in the project configuration if to
  use gc-go or gcc-go for some archs.
* Thu Aug 18 2016 jmassaguerpla@suse.com
- use gcc6-go instead of gcc5-go (bsc#988408)
- build ppc64le with gc-go because this version builds with gc-go 1.6
* Thu Aug 18 2016 cbrauner@suse.de
- bump git commit id to the one required by docker v1.12.0 (bsc#995058)
- run unit tests during package build
- remove seccomp-use-pkg-config.patch
  The patch is now upstream.
- remove GO_BUILD_FLAGS macro and substitute with BUILDFLAGS env variable to
  allow for easier string appending.
- only run unit test on architectures that provide the go list and go test tools
* Wed Aug 17 2016 cbrauner@suse.de
- Add runc-test package which contains the source code and the test. This
  package will be used to run the integration tests.
- Simplify package build and check sections: Instead of symlinking we default to
  cp -avr. go list gets confused by symlinks hence, we need to copy the source
  code anyway if we want to run unit tests during package build at some point.
* Fri Apr 29 2016 asarai@suse.de
  * Update to runC 0.1.1. (bsc#989566 FATE#320763) Changelog from upstream:
  This release includes a bug fix for adding the selinux mount label in the specification.
* Tue Apr 19 2016 asarai@suse.de
  * Don't use gcc-go for aarch64, since gc has grown support for it and is more
  stable.
* Fri Apr 15 2016 asarai@suse.de
  * Disable seccomp entirely for aarch64 builds, since it is not provided on all
  SUSE platforms.
* Wed Apr 13 2016 asarai@suse.de
  * Update to runC 0.1.0. Changelog from upstream:
  This release updates runc to the OCI runtime specification v0.5.0 and includes
  various fixes and features.
  Features:
  + cgroups: pid limits and stats
  + cgroups: kmem stats
  + systemd cgroup support
  + libcontainer specconv package
  + no pivot root option
  + numeric ids are treated as uid/gid
  + hook improvements
  Bug Fixes:
  * log flushing
  * atomic pid file creation
  * init error recovery
  * seccomp logging removed
  * delete container on aborted start
  * /dev bind mount handling
* Wed Mar 30 2016 asarai@suse.de
  * Install to /usr/sbin.  https://github.com/opencontainers/runc/pull/702
* Sun Mar 27 2016 asarai@suse.de
  * Added runC man pages.
  * Recommended criu, since it's required for the checkpoint and restore
  functionality.
* Sun Mar 27 2016 asarai@suse.de
  * Small updates to method of compilation to better match Makefile.
* Mon Mar 21 2016 asarai@suse.de
  * Make compilation work on gcc-go only systems (ppc and s390).
* Mon Mar 21 2016 asarai@suse.de
  * initial import of runC 0.0.9
  * add patch seccomp-use-pkg-config.patch which allows us to build runC, since
  they assume that the seccomp.h file lives at /usr/include/seccomp.h.
